@@ -16,6 +16,8 @@
 //! mathematical expressions in string form to float.
 
 use crate::CalculatorError;
+#[cfg(feature = "json_schema")]
+use schemars::schema::*;
 use serde::de::{Deserializer, Error, Visitor};
 use serde::ser::Serializer;
 use serde::{Deserialize, Serialize};
@@ -35,6 +37,7 @@ static RTOL: f64 = 1e-8;
 /// * `Str` - String instance
 ///
 #[derive(Debug, Clone, PartialEq)]
+// #[cfg_attr(feature = "json_schema", derive(schemars::JsonSchema))]
 pub enum CalculatorFloat {
     /// Floating point value
     Float(f64),
@@ -42,6 +45,20 @@ pub enum CalculatorFloat {
     Str(String),
 }
 
+#[cfg(feature = "json_schema")]
+impl schemars::JsonSchema for CalculatorFloat {
+    fn schema_name() -> String {
+        "CalculatorFloat".to_string()
+    }
+
+    fn json_schema(gen: &mut schemars::gen::SchemaGenerator) -> Schema {
+        let mut return_schema = SchemaObject::default();
+        return_schema.subschemas().one_of =
+            Some(vec![<f64>::json_schema(gen), <String>::json_schema(gen)]);
+        return_schema.into()
+    }
+}
+
 // Implementing serde serialization
 // writing directly to string or f64.
 impl Serialize for CalculatorFloat {
@@ -1062,6 +1079,8 @@ impl ops::Neg for CalculatorFloat {
 #[cfg(test)]
 mod tests {
     use super::CalculatorFloat;
+    #[cfg(feature = "json_schema")]
+    use schemars::schema_for;
     use serde_test::{assert_tokens, Configure, Token};
     use std::convert::TryFrom;
 
@@ -1086,6 +1105,61 @@ mod tests {
         assert_tokens(&x.readable(), &[Token::F64(0.0)]);
     }
 
+    #[test]
+    fn ser_de_string_compact() {
+        let x = CalculatorFloat::from("test+(1/3)");
+        assert_tokens(
+            &x.compact(),
+            &[
+                Token::NewtypeVariant {
+                    name: "CalculatorFloat",
+                    variant: "Str",
+                },
+                Token::String("test+(1/3)"),
+            ],
+        );
+    }
+
+    // Test the serialization/deserialization of CalculatorFloat from float
+    #[test]
+    fn ser_de_float_compact() {
+        let x = CalculatorFloat::from(3.0);
+        assert_tokens(
+            &x.compact(),
+            &[
+                Token::NewtypeVariant {
+                    name: "CalculatorFloat",
+                    variant: "Float",
+                },
+                Token::F64(3.0),
+            ],
+        );
+    }
+
+    // Test the serialization/deserialization of CalculatorFloat from integer
+    #[test]
+    fn ser_de_int_compact() {
+        let x = CalculatorFloat::from(0);
+        assert_tokens(
+            &x.compact(),
+            &[
+                Token::NewtypeVariant {
+                    name: "CalculatorFloat",
+                    variant: "Float",
+                },
+                Token::F64(0.0),
+            ],
+        );
+    }
+
+    #[cfg(feature = "json_schema")]
+    #[test]
+    fn test_json_schema_support() {
+        let schema = schema_for!(CalculatorFloat);
+        let serialized = serde_json::to_string(&schema).unwrap();
+        assert_eq!(serialized.as_str(), "{\"$schema\":\"http://json-schema.org/draft-07/schema#\",\"title\":\"CalculatorFloat\",\"oneOf\":[{\"type\":\"number\",\"format\":\"double\"},{\"type\":\"string\"}]}");
+    }
+
     // Test the initialisation of CalculatorFloat from all possible input types
     #[test]
     fn from() {
@@ -1221,7 +1295,7 @@ mod tests {
         // Test simple add function: x + y
         let mut x3 = CalculatorFloat::from(3);
         let x2 = CalculatorFloat::from(2.0);
-        if let CalculatorFloat::Float(y) = x3.clone() + x2.clone() {
+        if let CalculatorFloat::Float(y) = x3.clone() + x2 {
             assert!((y - 5.0).abs() < f64::EPSILON)
         }
         if let CalculatorFloat::Float(y) = x3.clone() + 2 {
@@ -1232,7 +1306,7 @@ mod tests {
         }
 
         let x2 = CalculatorFloat::from(0.0);
-        if let CalculatorFloat::Str(y) = x2.clone() + "3t" {
+        if let CalculatorFloat::Str(y) = x2 + "3t" {
             assert_eq!(y, "3t")
         }
 
@@ -1240,7 +1314,7 @@ mod tests {
         if let CalculatorFloat::Str(y) = x2.clone() + 0.0 {
             assert_eq!(y, "3t")
         }
-        if let CalculatorFloat::Str(y) = x2.clone() + "2x" {
+        if let CalculatorFloat::Str(y) = x2 + "2x" {
             assert_eq!(y, "(3t + 2x)")
         }
 
@@ -1381,7 +1455,7 @@ mod tests {
         // Test simple multiply function: x * y
         let mut x3 = CalculatorFloat::from(3);
         let x2 = CalculatorFloat::from(3.0);
-        assert_eq!(x3.clone() * x2.clone(), CalculatorFloat::Float(9.0));
+        assert_eq!(x3.clone() * x2, CalculatorFloat::Float(9.0));
         assert_eq!(x3.clone() * 3, CalculatorFloat::Float(9.0));
         assert_eq!(x3.clone() * 3.0, CalculatorFloat::Float(9.0));
         assert_eq!(
@@ -1450,13 +1524,10 @@ mod tests {
         assert_eq!(x3.clone() - x2.clone(), CalculatorFloat::Float(0.0));
         assert_eq!(x3.clone() - 3, CalculatorFloat::Float(0.0));
         assert_eq!(x3.clone() - 3.0, CalculatorFloat::Float(0.0));
-        assert_eq!(
-            x3.clone() - "x",
-            CalculatorFloat::Str(String::from("(3e0 - x)"))
-        );
+        assert_eq!(x3 - "x", CalculatorFloat::Str(String::from("(3e0 - x)")));
 
         let x3 = CalculatorFloat::from(0.0);
-        assert_eq!(x3.clone() - "x", CalculatorFloat::Str(String::from("(-x)")));
+        assert_eq!(x3 - "x", CalculatorFloat::Str(String::from("(-x)")));
 
         let mut x3s = CalculatorFloat::from("3t");
         assert_eq!(
@@ -1500,10 +1571,10 @@ mod tests {
     #[test]
     fn neg() {
         let x3 = CalculatorFloat::from(3);
-        let x2 = -x3.clone();
+        let x2 = -x3;
         assert_eq!(x2, CalculatorFloat::Float(-3.0));
         let x3s = CalculatorFloat::from("3t");
-        let x2 = -x3s.clone();
+        let x2 = -x3s;
         assert_eq!(x2, CalculatorFloat::Str(String::from("(-3t)")));
     }
 
@@ -1643,10 +1714,10 @@ mod tests {
     fn isclose() {
         let x2 = CalculatorFloat::from(-3);
         let x3 = CalculatorFloat::from("-3t");
-        assert_eq!(x2.isclose(-3.000000001), true);
-        assert_eq!(x3.isclose("-3.000000001t"), false);
-        assert_eq!(x3.isclose(-3.000000001), false);
-        assert_eq!(x2.isclose("-3.000000001t"), false);
+        assert!(x2.isclose(-3.000000001));
+        assert!(!x3.isclose("-3.000000001t"));
+        assert!(!x3.isclose(-3.000000001));
+        assert!(!x2.isclose("-3.000000001t"));
     }
 
     // Test the adding with reference input functionality of CalculatorFloat
